=== modified file '.pc/fix-ubuntu-tests.patch/tests/test_keystoneclient.py'
--- .pc/fix-ubuntu-tests.patch/tests/test_keystoneclient.py	2013-08-09 10:06:27 +0000
+++ .pc/fix-ubuntu-tests.patch/tests/test_keystoneclient.py	2013-10-17 13:52:59 +0000
@@ -379,6 +379,52 @@
                           client.tokens.authenticate,
                           token=token_id)
 
+    def test_disable_tenant_invalidates_token(self):
+        from keystoneclient import exceptions as client_exceptions
+
+        admin_client = self.get_client(admin=True)
+        foo_client = self.get_client(self.user_foo)
+        tenant_bar = admin_client.tenants.get(self.tenant_bar['id'])
+
+        # Disable the tenant.
+        tenant_bar.update(enabled=False)
+
+        # Test that the token has been removed.
+        self.assertRaises(client_exceptions.Unauthorized,
+                          foo_client.tokens.authenticate,
+                          token=foo_client.auth_token)
+
+        # Test that the user access has been disabled.
+        self.assertRaises(client_exceptions.Unauthorized,
+                          self.get_client,
+                          self.user_foo)
+
+    def test_delete_tenant_invalidates_token(self):
+        from keystoneclient import exceptions as client_exceptions
+
+        admin_client = self.get_client(admin=True)
+        foo_client = self.get_client(self.user_foo, self.tenant_bar)
+        tenant_bar = admin_client.tenants.get(self.tenant_bar['id'])
+
+        # Delete the tenant.
+        tenant_bar.delete()
+
+        # Test that the token has been removed.
+        self.assertRaises(client_exceptions.Unauthorized,
+                          foo_client.tokens.authenticate,
+                          token=foo_client.auth_token)
+
+        # Test that the user access has been disabled.
+        """
+        # FIXME(dolph): this assertion should not be skipped, but appears to be
+        #               an unrelated bug? auth succeeds, even though tenant_bar
+        #               was deleted
+        self.assertRaises(client_exceptions.Unauthorized,
+                          self.get_client,
+                          self.user_foo,
+                          self.tenant_bar)
+        """
+
     def test_disable_user_invalidates_token(self):
         from keystoneclient import exceptions as client_exceptions
 
@@ -1144,6 +1190,12 @@
         """Due to lack of endpoint CRUD"""
         raise nose.exc.SkipTest('N/A')
 
+    def test_disable_tenant_invalidates_token(self):
+        raise self.skipTest('N/A')
+
+    def test_delete_tenant_invalidates_token(self):
+        raise self.skipTest('N/A')
+
 
 class Kc11TestCase(CompatTestCase, KeystoneClientTests):
     def get_checkout(self):

=== modified file 'AUTHORS'
--- AUTHORS	2013-08-09 10:06:27 +0000
+++ AUTHORS	2013-10-17 13:52:59 +0000
@@ -115,6 +115,7 @@
 Mohammed Naser <mnaser@vexxhost.com>
 monsterxx03 <xyj.asmy@gmail.com>
 Monty Taylor <mordred@inaugust.com>
+Morgan Fainberg <m@metacloud.com>
 Nachiappan VR N <nachiappan.veerappan-nachiappan@hp.com>
 Nathanael Burton <nathanael.i.burton.work@gmail.com>
 OpenStack Jenkins <jenkins@openstack.org>

=== modified file 'ChangeLog'
--- ChangeLog	2013-08-09 10:06:27 +0000
+++ ChangeLog	2013-10-17 13:52:59 +0000
@@ -1,3 +1,138 @@
+commit 9666fc0e14d060afe05e2c2516f0c91aac3f4d15
+Merge: afbc75b 801a815
+Author: Jenkins <jenkins@review.openstack.org>
+Date:   Tue Oct 8 21:46:16 2013 +0000
+
+    Merge "Fixes for Active Directory" into stable/grizzly
+
+commit afbc75b08add8fb5201f4ca7ccf1b7353fab138c
+Merge: 0876ea2 c70f8c6
+Author: Jenkins <jenkins@review.openstack.org>
+Date:   Tue Sep 17 12:13:07 2013 +0000
+
+    Merge "Revoke user tokens when disabling/delete a project" into stable/grizzly
+
+commit 0876ea20c1076f806285a543522ba99e95703316
+Merge: 775d7a7 53ff445
+Author: Jenkins <jenkins@review.openstack.org>
+Date:   Mon Sep 16 11:27:25 2013 +0000
+
+    Merge "Use joins instead of multiple lookups in groups sql" into stable/grizzly
+
+commit c70f8c61d50c2358d712b365bec4a8f288314b54
+Author: Dolph Mathews <dolph.mathews@rackspace.com>
+Date:   Thu Sep 12 17:02:26 2013 -0500
+
+    Revoke user tokens when disabling/delete a project
+    
+    - Revoke tokens scoped to all users from a project when disabling or
+      deleting the project.
+    - Fix provided by chmouel
+    
+    Closes-Bug: #1179955
+    Change-Id: I8ab4713d513b26ced6c37ed026cec9e2df78a5e9
+
+ keystone/common/controller.py    |    6 ++++
+ keystone/identity/controllers.py |   16 ++++++++++
+ tests/test_keystoneclient.py     |   52 ++++++++++++++++++++++++++++++++
+ tests/test_v3_auth.py            |   61 ++++++++++++++++++++++++++++++++++++++
+ 4 files changed, 135 insertions(+)
+
+commit 775d7a7802bd5cf49880c8a76e9a175e8470adba
+Author: Morgan Fainberg <m@metacloud.com>
+Date:   Fri Aug 23 14:53:26 2013 -0700
+
+    Fix and test token revocation list API
+    
+    Change-Id: I6c60bf2aecc7c9353e837e59a4e09860d049e0f5
+
+ keystone/token/backends/kvs.py      |    2 +-
+ keystone/token/backends/memcache.py |   12 +++++----
+ tests/test_backend.py               |   47 +++++++++++++++++++++++++++++------
+ 3 files changed, 48 insertions(+), 13 deletions(-)
+
+commit 53ff4451c31c888e6cafac9b3a44599601cf6bdc
+Author: Jamie Lennox <jlennox@redhat.com>
+Date:   Mon Aug 19 15:00:03 2013 +1000
+
+    Use joins instead of multiple lookups in groups sql
+    
+    Currently to determine a user's groups (or vice versa) we do
+    - 1 lookup for the user.
+    - 1 lookup for the association table.
+    - n lookups for the n groups the user has.
+    
+    The association and groups lookup are replaced with a join query.
+    
+    Fixes: bug 1218675
+    Change-Id: I560659eda1a30635399829e1c86f06734c90f9e2
+    (cherry picked from commit 6f9b75e2c3bbbd7dfc5e2d414d7271bb4f8bcf71)
+
+ keystone/identity/backends/sql.py |   15 +++++++--------
+ 1 file changed, 7 insertions(+), 8 deletions(-)
+
+commit 801a81506e688bca800bd4164b4210fdcc5fee05
+Author: Brant Knudson <bknudson@us.ibm.com>
+Date:   Thu Aug 8 15:36:20 2013 -0500
+
+    Fixes for Active Directory
+    
+    The LDAP Identity backend was not properly using the
+    user_enabled_default option as a string. This caused
+    operations to fail with
+    
+     TypeError: unsupported operand type(s) for &: 'str' and 'int'
+    
+    Also, fetching users using the LDAP Identity backend would fail with
+    
+     KeyError: 'enabled'
+    
+    from _ldap_res_to_model when user_enabled_mask is not 0.
+    
+    Closes-Bug: #1210175
+    (cherry picked from commit 68f38a65c60485c34474e490d649b328421e10f5)
+    (cherry picked from commit 116897786dbb8473154ec85a01b019af8106a1f4)
+    (cherry picked from commit 54178b735dea4dfee4578caa95cb3ae704afef07)
+    (cherry picked from commit 87ababb38506a1a51b9e38fc343dd3b46c828a80)
+    (cherry picked from commit 781c65b72b78bd1e2b1d93db029d6b0c6fbc2050)
+    
+    Change-Id: Ic01aa505d867c1de30e2a1ed7c79ff1478e213ef
+
+ keystone/identity/backends/ldap/core.py |    7 +++--
+ tests/_ldap_livetest.py                 |    8 ++++--
+ tests/test_backend_ldap.py              |   47 +++++++++++++++++++++++++------
+ 3 files changed, 47 insertions(+), 15 deletions(-)
+
+commit 6792499f76545d9789472c0f85e163fec0c502ac
+Author: Jamie Lennox <jlennox@redhat.com>
+Date:   Fri Aug 23 15:06:58 2013 +1000
+
+    Add netaddr to test requirements
+    
+    Because of the way we do old keystoneclient tests we can end up with the
+    situation where an old keystone uses a newer keystoneclient without
+    doing an install and so misses out on the newer dependencies.
+    
+    This patch is not a solution to the underlying problem, but should
+    prevent current gate failings.
+    
+    Fixes: bug 1212939
+    Change-Id: I25eca24c6820611a5937a1f709ba42620b70b304
+
+ tools/test-requires |    1 +
+ 1 file changed, 1 insertion(+)
+
+commit 56851fa3ffa95831c7352b6f8e70ede77d6b2fdd
+Author: Alan Pevec <apevec@redhat.com>
+Date:   Thu Aug 8 22:52:12 2013 +0200
+
+    Bump stable/grizzly next version to 2013.1.4
+    
+    Change-Id: I817767bcb618edf94363aa34dd0eb6ce897ce141
+
+ setup.py |    2 +-
+ 1 file changed, 1 insertion(+), 1 deletion(-)
+
 commit f60f742a78f3e62017279e8f87353568a70cfc6c
 Author: Dolph Mathews <dolph.mathews@rackspace.com>
 Date:   Mon Jun 3 14:21:36 2013 -0500

=== modified file 'PKG-INFO'
--- PKG-INFO	2013-08-09 10:06:27 +0000
+++ PKG-INFO	2013-10-17 13:52:59 +0000
@@ -1,6 +1,6 @@
 Metadata-Version: 1.1
 Name: keystone
-Version: 2013.1.3
+Version: 2013.1.4
 Summary: Authentication service for OpenStack
 Home-page: http://www.openstack.org
 Author: OpenStack, LLC.

=== modified file 'debian/changelog'
--- debian/changelog	2013-08-09 10:06:27 +0000
+++ debian/changelog	2013-10-17 13:52:59 +0000
@@ -1,3 +1,16 @@
+keystone (1:2013.1.4-0ubuntu1) raring-proposed; urgency=low
+
+  * Resynchronize with stable/grizzly (9666fc0) (LP: #1241202):
+    - [6792499] periodic-keystone-python27-stable-grizzly fails due to"No
+      module named netaddr"   LP: 1212939
+    - [775d7a7] Fix and test token revocation list API
+    - [0876ea2] N+1 lookups in groups SQL LP: 1218675
+    - [afbc75b] Disabling a tenant would not disable a user token LP: 1179955
+    - [9666fc0] User operations with LDAP Identity and
+      enabled_mask/user_enabled_default fail LP: 1210175
+
+ -- Adam Gandelman <adamg@ubuntu.com>  Thu, 17 Oct 2013 13:52:59 -0700
+
 keystone (1:2013.1.3-0ubuntu1) raring-proposed; urgency=low
 
   * Dropped patches, applied upstream:

=== modified file 'keystone.egg-info/PKG-INFO'
--- keystone.egg-info/PKG-INFO	2013-08-09 10:06:27 +0000
+++ keystone.egg-info/PKG-INFO	2013-10-17 13:52:59 +0000
@@ -1,6 +1,6 @@
 Metadata-Version: 1.1
 Name: keystone
-Version: 2013.1.3
+Version: 2013.1.4
 Summary: Authentication service for OpenStack
 Home-page: http://www.openstack.org
 Author: OpenStack, LLC.

=== modified file 'keystone/common/controller.py'
--- keystone/common/controller.py	2013-08-09 10:06:27 +0000
+++ keystone/common/controller.py	2013-10-17 13:52:59 +0000
@@ -171,6 +171,12 @@
                                           trust['trustee_user_id'],
                                           trust['id'])
 
+    def _delete_tokens_for_project(self, context, project_id):
+        for user_ref in self.identity_api.get_project_users(
+                context, project_id):
+            self._delete_tokens_for_user(
+                context, user_ref['id'], project_id=project_id)
+
     def _require_attribute(self, ref, attr):
         """Ensures the reference contains the specified attribute."""
         if ref.get(attr) is None or ref.get(attr) == '':

=== modified file 'keystone/identity/backends/ldap/core.py'
--- keystone/identity/backends/ldap/core.py	2013-08-09 10:06:27 +0000
+++ keystone/identity/backends/ldap/core.py	2013-10-17 13:52:59 +0000
@@ -430,14 +430,15 @@
     def _ldap_res_to_model(self, res):
         obj = super(UserApi, self)._ldap_res_to_model(res)
         if self.enabled_mask != 0:
-            obj['enabled_nomask'] = obj['enabled']
-            obj['enabled'] = ((obj['enabled'] & self.enabled_mask) !=
+            enabled = int(obj.get('enabled', self.enabled_default))
+            obj['enabled_nomask'] = enabled
+            obj['enabled'] = ((enabled & self.enabled_mask) !=
                               self.enabled_mask)
         return obj
 
     def mask_enabled_attribute(self, values):
         value = values['enabled']
-        values.setdefault('enabled_nomask', self.enabled_default)
+        values.setdefault('enabled_nomask', int(self.enabled_default))
         if value != ((values['enabled_nomask'] & self.enabled_mask) !=
                      self.enabled_mask):
             values['enabled_nomask'] ^= self.enabled_mask

=== modified file 'keystone/identity/backends/sql.py'
--- keystone/identity/backends/sql.py	2013-08-09 10:06:27 +0000
+++ keystone/identity/backends/sql.py	2013-10-17 13:52:59 +0000
@@ -721,18 +721,17 @@
     def list_groups_for_user(self, user_id):
         session = self.get_session()
         self.get_user(user_id)
-        query = session.query(UserGroupMembership)
-        query = query.filter_by(user_id=user_id)
-        membership_refs = query.all()
-        return [self.get_group(x.group_id) for x in membership_refs]
+        query = session.query(Group).join(UserGroupMembership)
+        query = query.filter(UserGroupMembership.user_id == user_id)
+        return [g.to_dict() for g in query]
 
     def list_users_in_group(self, group_id):
         session = self.get_session()
         self.get_group(group_id)
-        query = session.query(UserGroupMembership)
-        query = query.filter_by(group_id=group_id)
-        membership_refs = query.all()
-        return [self.get_user(x.user_id) for x in membership_refs]
+        query = session.query(User).join(UserGroupMembership)
+        query = query.filter(UserGroupMembership.group_id == group_id)
+
+        return [identity.filter_user(u.to_dict()) for u in query]
 
     def delete_user(self, user_id):
         session = self.get_session()

=== modified file 'keystone/identity/controllers.py'
--- keystone/identity/controllers.py	2013-08-09 10:06:27 +0000
+++ keystone/identity/controllers.py	2013-10-17 13:52:59 +0000
@@ -111,12 +111,20 @@
         # be specifying that
         clean_tenant = tenant.copy()
         clean_tenant.pop('domain_id', None)
+
+        # If the project has been disabled (or enabled=False) we are
+        # deleting the tokens for that project.
+        if not tenant.get('enabled', True):
+            self._delete_tokens_for_project(context, tenant_id)
+
         tenant_ref = self.identity_api.update_project(
             context, tenant_id, clean_tenant)
         return {'tenant': tenant_ref}
 
     def delete_project(self, context, tenant_id):
         self.assert_admin(context)
+        # Delete all tokens belonging to the users for that project
+        self._delete_tokens_for_project(context, tenant_id)
         self.identity_api.delete_project(context, tenant_id)
 
     def get_project_users(self, context, tenant_id, **kw):
@@ -571,6 +579,10 @@
     def update_project(self, context, project_id, project):
         self._require_matching_id(project_id, project)
 
+        # The project was disabled so we delete the tokens
+        if not project.get('enabled', True):
+            self._delete_tokens_for_project(context, project_id)
+
         ref = self.identity_api.update_project(context, project_id, project)
         return ProjectV3.wrap_member(context, ref)
 
@@ -579,6 +591,10 @@
         for cred in self.identity_api.list_credentials(context):
             if cred['project_id'] == project_id:
                 self.identity_api.delete_credential(context, cred['id'])
+
+        # Delete all tokens belonging to the users for that project
+        self._delete_tokens_for_project(context, project_id)
+
         # Finally delete the project itself - the backend is
         # responsible for deleting any role assignments related
         # to this project

=== modified file 'keystone/token/backends/kvs.py'
--- keystone/token/backends/kvs.py	2013-03-22 08:16:09 +0000
+++ keystone/token/backends/kvs.py	2013-10-17 13:52:59 +0000
@@ -111,7 +111,7 @@
             if not token.startswith('revoked-token-'):
                 continue
             record = {}
-            record['id'] = token_ref['id']
+            record['id'] = token[len('revoked-token-'):]
             record['expires'] = token_ref['expires']
             tokens.append(record)
         return tokens

=== modified file 'keystone/token/backends/memcache.py'
--- keystone/token/backends/memcache.py	2013-05-14 07:49:15 +0000
+++ keystone/token/backends/memcache.py	2013-10-17 13:52:59 +0000
@@ -84,8 +84,9 @@
                         raise exception.UnexpectedError(msg)
         return copy.deepcopy(data_copy)
 
-    def _add_to_revocation_list(self, data):
-        data_json = jsonutils.dumps(data)
+    def _add_to_revocation_list(self, token_id, token_data):
+        data_json = jsonutils.dumps({'id': token_id,
+                                     'expires': token_data['expires']})
         if not self.client.append(self.revocation_key, ',%s' % data_json):
             if not self.client.add(self.revocation_key, data_json):
                 if not self.client.append(self.revocation_key,
@@ -95,10 +96,11 @@
 
     def delete_token(self, token_id):
         # Test for existence
-        data = self.get_token(token.unique_id(token_id))
-        ptk = self._prefix_token_id(token.unique_id(token_id))
+        token_id = token.unique_id(token_id)
+        data = self.get_token(token_id)
+        ptk = self._prefix_token_id(token_id)
         result = self.client.delete(ptk)
-        self._add_to_revocation_list(data)
+        self._add_to_revocation_list(token_id, data)
         return result
 
     def list_tokens(self, user_id, tenant_id=None, trust_id=None):

=== modified file 'setup.py'
--- setup.py	2013-08-09 10:06:27 +0000
+++ setup.py	2013-10-17 13:52:59 +0000
@@ -26,7 +26,7 @@
 
 setuptools.setup(
     name=project,
-    version=setup.get_version(project, '2013.1.3'),
+    version=setup.get_version(project, '2013.1.4'),
     description="Authentication service for OpenStack",
     license='Apache License (2.0)',
     author='OpenStack, LLC.',

=== modified file 'tests/_ldap_livetest.py'
--- tests/_ldap_livetest.py	2013-03-22 08:16:09 +0000
+++ tests/_ldap_livetest.py	2013-10-17 13:52:59 +0000
@@ -92,9 +92,6 @@
     def tearDown(self):
         test.TestCase.tearDown(self)
 
-    def test_user_enable_attribute_mask(self):
-        raise nose.exc.SkipTest('Test is for Active Directory Only')
-
     def test_ldap_dereferencing(self):
         alt_users_ldif = {'objectclass': ['top', 'organizationalUnit'],
                           'ou': 'alt_users'}
@@ -163,3 +160,8 @@
                                                alias_dereferencing=deref)
         self.assertEqual(ldap.DEREF_SEARCHING,
                          ldap_wrapper.conn.get_option(ldap.OPT_DEREF))
+
+    def test_user_enable_attribute_mask(self):
+        CONF.ldap.user_enabled_emulation = False
+        CONF.ldap.user_enabled_attribute = 'employeeType'
+        super(LiveLDAPIdentity, self).test_user_enable_attribute_mask()

=== modified file 'tests/test_backend.py'
--- tests/test_backend.py	2013-08-09 10:06:27 +0000
+++ tests/test_backend.py	2013-10-17 13:52:59 +0000
@@ -14,10 +14,11 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import copy
 import datetime
 import default_fixtures
+import hashlib
 import uuid
-import nose.exc
 
 from keystone.catalog import core
 from keystone import config
@@ -2065,17 +2066,19 @@
                 'trust_id': None,
                 'user': {'id': 'testuserid'}}
         data_ref = self.token_api.create_token(token_id, data)
-        expires = data_ref.pop('expires')
-        data_ref.pop('user_id')
+        data_ref_copy = copy.deepcopy(data_ref)
+        expires = data_ref_copy.pop('expires')
+        data_ref_copy.pop('user_id')
         self.assertTrue(isinstance(expires, datetime.datetime))
-        self.assertDictEqual(data_ref, data)
+        self.assertDictEqual(data_ref_copy, data)
 
         new_data_ref = self.token_api.get_token(token_id)
-        expires = new_data_ref.pop('expires')
-        new_data_ref.pop('user_id')
+        new_data_ref_copy = copy.deepcopy(new_data_ref)
+        expires = new_data_ref_copy.pop('expires')
+        new_data_ref_copy.pop('user_id')
 
         self.assertTrue(isinstance(expires, datetime.datetime))
-        self.assertEquals(new_data_ref, data)
+        self.assertEquals(new_data_ref_copy, data)
 
         self.token_api.delete_token(token_id)
         self.assertRaises(exception.TokenNotFound,
@@ -2248,6 +2251,36 @@
         self.check_list_revoked_tokens([self.delete_token()
                                         for x in xrange(2)])
 
+    def test_predictable_revoked_pki_token_id(self):
+        # NOTE(dolph): _create_token_id() includes 'MII' as a prefix of the
+        #               returned token str in master, but not in grizzly.
+        #               revising _create_token_id() in grizzly to include the
+        #               previx breaks several other tests here
+        token_id = 'MII' + self._create_token_id()
+        token_id_hash = hashlib.md5(token_id).hexdigest()
+        token = {'user': {'id': uuid.uuid4().hex}}
+
+        self.token_api.create_token(token_id, token)
+        self.token_api.delete_token(token_id)
+
+        revoked_ids = [x['id'] for x in self.token_api.list_revoked_tokens()]
+        self.assertIn(token_id_hash, revoked_ids)
+        self.assertNotIn(token_id, revoked_ids)
+        for t in self.token_api.list_revoked_tokens():
+            self.assertIn('expires', t)
+
+    def test_predictable_revoked_uuid_token_id(self):
+        token_id = uuid.uuid4().hex
+        token = {'user': {'id': uuid.uuid4().hex}}
+
+        self.token_api.create_token(token_id, token)
+        self.token_api.delete_token(token_id)
+
+        revoked_ids = [x['id'] for x in self.token_api.list_revoked_tokens()]
+        self.assertIn(token_id, revoked_ids)
+        for t in self.token_api.list_revoked_tokens():
+            self.assertIn('expires', t)
+
 
 class TrustTests(object):
     def create_sample_trust(self, new_id):

=== modified file 'tests/test_backend_ldap.py'
--- tests/test_backend_ldap.py	2013-08-09 10:06:27 +0000
+++ tests/test_backend_ldap.py	2013-10-17 13:52:59 +0000
@@ -19,7 +19,6 @@
 import uuid
 import nose.exc
 
-from keystone.common import ldap as ldap_common
 from keystone.common.ldap import fakeldap
 from keystone import config
 from keystone import exception
@@ -318,25 +317,55 @@
         self.assertNotIn('name', role_ref)
 
     def test_user_enable_attribute_mask(self):
-        CONF.ldap.user_enabled_attribute = 'enabled'
         CONF.ldap.user_enabled_mask = 2
-        CONF.ldap.user_enabled_default = 512
+        CONF.ldap.user_enabled_default = '512'
         self.clear_database()
         self.identity_api = identity.backends.ldap.Identity()
+        self.load_fixtures(default_fixtures)
+
+        ldap_ = self.identity_api.user.get_connection()
+
+        def get_enabled_vals():
+            user_dn = self.identity_api.user._id_to_dn_string('fake1')
+            enabled_attr_name = CONF.ldap.user_enabled_attribute
+
+            res = ldap_.search_s(user_dn,
+                                 ldap.SCOPE_BASE,
+                                 query='(sn=fake1)')
+            return res[0][1][enabled_attr_name]
+
         user = {'id': 'fake1', 'name': 'fake1', 'enabled': True}
-        self.identity_api.create_user('fake1', user)
+
+        user_ref = self.identity_api.create_user('fake1', user)
+
+        self.assertEqual(user_ref['enabled'], 512)
+        # TODO(blk-u): 512 seems wrong, should it be True?
+
+        enabled_vals = get_enabled_vals()
+        self.assertEqual(enabled_vals, [512])
+
         user_ref = self.identity_api.get_user('fake1')
-        self.assertEqual(user_ref['enabled'], True)
+        self.assertIs(user_ref['enabled'], True)
 
         user['enabled'] = False
-        self.identity_api.update_user('fake1', user)
+        user_ref = self.identity_api.update_user('fake1', user)
+        self.assertIs(user_ref['enabled'], False)
+
+        enabled_vals = get_enabled_vals()
+        self.assertEqual(enabled_vals, [514])
+
         user_ref = self.identity_api.get_user('fake1')
-        self.assertEqual(user_ref['enabled'], False)
+        self.assertIs(user_ref['enabled'], False)
 
         user['enabled'] = True
-        self.identity_api.update_user('fake1', user)
+        user_ref = self.identity_api.update_user('fake1', user)
+        self.assertIs(user_ref['enabled'], True)
+
+        enabled_vals = get_enabled_vals()
+        self.assertEqual(enabled_vals, [512])
+
         user_ref = self.identity_api.get_user('fake1')
-        self.assertEqual(user_ref['enabled'], True)
+        self.assertIs(user_ref['enabled'], True)
 
     def test_user_api_get_connection_no_user_password(self):
         """Don't bind in case the user and password are blank"""

=== modified file 'tests/test_keystoneclient.py'
--- tests/test_keystoneclient.py	2013-08-09 10:06:27 +0000
+++ tests/test_keystoneclient.py	2013-10-17 13:52:59 +0000
@@ -375,6 +375,52 @@
                           client.tokens.authenticate,
                           token=token_id)
 
+    def test_disable_tenant_invalidates_token(self):
+        from keystoneclient import exceptions as client_exceptions
+
+        admin_client = self.get_client(admin=True)
+        foo_client = self.get_client(self.user_foo)
+        tenant_bar = admin_client.tenants.get(self.tenant_bar['id'])
+
+        # Disable the tenant.
+        tenant_bar.update(enabled=False)
+
+        # Test that the token has been removed.
+        self.assertRaises(client_exceptions.Unauthorized,
+                          foo_client.tokens.authenticate,
+                          token=foo_client.auth_token)
+
+        # Test that the user access has been disabled.
+        self.assertRaises(client_exceptions.Unauthorized,
+                          self.get_client,
+                          self.user_foo)
+
+    def test_delete_tenant_invalidates_token(self):
+        from keystoneclient import exceptions as client_exceptions
+
+        admin_client = self.get_client(admin=True)
+        foo_client = self.get_client(self.user_foo, self.tenant_bar)
+        tenant_bar = admin_client.tenants.get(self.tenant_bar['id'])
+
+        # Delete the tenant.
+        tenant_bar.delete()
+
+        # Test that the token has been removed.
+        self.assertRaises(client_exceptions.Unauthorized,
+                          foo_client.tokens.authenticate,
+                          token=foo_client.auth_token)
+
+        # Test that the user access has been disabled.
+        """
+        # FIXME(dolph): this assertion should not be skipped, but appears to be
+        #               an unrelated bug? auth succeeds, even though tenant_bar
+        #               was deleted
+        self.assertRaises(client_exceptions.Unauthorized,
+                          self.get_client,
+                          self.user_foo,
+                          self.tenant_bar)
+        """
+
     def test_disable_user_invalidates_token(self):
         from keystoneclient import exceptions as client_exceptions
 
@@ -1147,6 +1193,12 @@
         """Due to lack of endpoint CRUD"""
         raise nose.exc.SkipTest('N/A')
 
+    def test_disable_tenant_invalidates_token(self):
+        raise self.skipTest('N/A')
+
+    def test_delete_tenant_invalidates_token(self):
+        raise self.skipTest('N/A')
+
 
 class Kc11TestCase(CompatTestCase, KeystoneClientTests):
     def get_checkout(self):

=== modified file 'tests/test_v3_auth.py'
--- tests/test_v3_auth.py	2013-08-09 10:06:27 +0000
+++ tests/test_v3_auth.py	2013-10-17 13:52:59 +0000
@@ -595,6 +595,67 @@
                   headers={'X-Subject-Token': token},
                   expected_status=204)
 
+    def test_disabling_project_revokes_token(self):
+        resp = self.post(
+            '/auth/tokens',
+            body=self.build_authentication_request(
+                user_id=self.user3['id'],
+                password=self.user3['password'],
+                project_id=self.projectA['id']))
+        token = resp.getheader('X-Subject-Token')
+
+        # confirm token is valid
+        self.head('/auth/tokens',
+                  headers={'X-Subject-Token': token},
+                  expected_status=204)
+
+        # disable the project, which should invalidate the token
+        self.patch(
+            '/projects/%(project_id)s' % {'project_id': self.projectA['id']},
+            body={'project': {'enabled': False}})
+
+        # user should no longer have access to the project
+        self.head('/auth/tokens',
+                  headers={'X-Subject-Token': token},
+                  expected_status=401)
+        resp = self.post(
+            '/auth/tokens',
+            body=self.build_authentication_request(
+                user_id=self.user3['id'],
+                password=self.user3['password'],
+                project_id=self.projectA['id']),
+            expected_status=401)
+
+    def test_deleting_project_revokes_token(self):
+        resp = self.post(
+            '/auth/tokens',
+            body=self.build_authentication_request(
+                user_id=self.user3['id'],
+                password=self.user3['password'],
+                project_id=self.projectA['id']))
+        token = resp.getheader('X-Subject-Token')
+
+        # confirm token is valid
+        self.head('/auth/tokens',
+                  headers={'X-Subject-Token': token},
+                  expected_status=204)
+
+        # delete the project, which should invalidate the token
+        self.delete(
+            '/projects/%(project_id)s' % {'project_id': self.projectA['id']})
+
+        # user should no longer have access to the project
+        self.head('/auth/tokens',
+                  headers={'X-Subject-Token': token},
+                  expected_status=401)
+        resp = self.post(
+            '/auth/tokens',
+            body=self.build_authentication_request(
+                user_id=self.user3['id'],
+                password=self.user3['password'],
+                project_id=self.projectA['id']),
+            expected_status=401)
+
     def test_deleting_group_grant_revokes_tokens(self):
         """Test deleting a group grant revokes tokens.
 

=== modified file 'tools/test-requires'
--- tools/test-requires	2013-06-16 21:45:44 +0000
+++ tools/test-requires	2013-10-17 13:52:59 +0000
@@ -25,6 +25,7 @@
 httplib2 # keystoneclient <0.2.1
 requests>=1.0.0 # replaces httplib2 in keystoneclient >=0.2.1
 keyring
+netaddr
 
 # swift_auth test dependencies
 http://tarballs.openstack.org/swift/swift-stable-grizzly.tar.gz#egg=swift

